%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************
function [setOfHYPRFrames,setOfHYPRFrames_noise,status] = FBP(hObject)

    function make_one_projection
        numberOfProjectionsInCurrentFrame = numberOfProjectionsInCurrentFrame+1;

        i = projectionNumber;
        display_projection_vector(server_1_handles.server_1_projection_direction_axes,...
            client_handles.userData.angles(projectionNumber));

        update_UI_counters_at_start_of_projection(...
            server_1_handles,...
            client_handles,...
            hyprFrameNumber,...
            numberOfProjectionsInCurrentFrame,...
            projectionNumber);


        HYPRcommonLoopProcess(client_handles,server_1_handles,i) ;

        from = 1;
        to   = i;
        FILTERED = true;
        MODE = 0;  %from g
        noise_process = false;
        fbp_total = doBackProjection( client_handles, FILTERED, from, to, MODE, noise_process);

        from = 1;
        to   = i;
        FILTERED = false;
        MODE = 0;  %from g
        noise_process = false;
        bp_total = doBackProjection( client_handles, FILTERED, from, to, MODE, noise_process);

        if get(server_1_handles.turnOnProjectionUpdatesTag,'Value')==1
            axes(server_1_handles.currentPAxes);
            imshow(fbp_total,[ ]); %colorbar;

            setXandTitleLabels(sprintf('FBP[%d],mean=%6.2f,max=%6.2f',i,mean(fbp_total(:)),max(fbp_total(:))),...
                '',...
                7,client_handles.userData.xlabelOffset);

            axes(server_1_handles.currentPCAxes);
            imshow(bp_total,[ ]); %colorbar;
            setXandTitleLabels(sprintf('BP[%d],mean=%6.2f,max=%6.2f',i,mean(bp_total(:)),max(bp_total(:))),...
                '',...
                7,client_handles.userData.xlabelOffset);
        end

        if 0
            if numberOfProjectionsInCurrentFrame == client_handles.userData.nProjectionsPerFrame
                numberOfProjectionsInCurrentFrame = 0;

                from = 1;
                to   = hyprFrameNumber*client_handles.userData.nProjectionsPerFrame;
                FILTERED = true;
                MODE = 0;  %from g
                noise_process = false;
                fbp_total = doBackProjection( client_handles, FILTERED, from, to, MODE, noise_process);
                noise_process = true;
                fbp_total_noise = doBackProjection( client_handles, FILTERED, from, to, MODE, noise_process);

                [currentHYPRframe,...
                    running_psnr_vector,...
                    totalMeanAbsDiff] = generate_FBP_HYPR_frame(...
                    fbp_total,...
                    client_handles,...
                    server_1_handles,...
                    hyprFrameNumber,...
                    running_psnr_vector,...
                    totalMeanAbsDiff,...
                    projectionNumber);

                setOfHYPRFrames(:,:,hyprFrameNumber) = currentHYPRframe;
                setOfHYPRFrames_noise(:,:,hyprFrameNumber) =  fbp_total_noise;
                hyprFrameNumber = hyprFrameNumber + 1;
                %fbp = zeros(client_handles.userData.sz,client_handles.userData.sz);
                %bp = zeros(client_handles.userData.sz,client_handles.userData.sz);
            end
        end
        drawnow;
    end

client_handles   = guidata(hObject);
server_1_handles = guidata(client_handles.userData.server_1_handle);

running_psnr_vector = zeros(client_handles.userData.nTimeFrames,1);

updateLogFile(sprintf('frame\trelative RMSE\tAbsDifference\n'),client_handles.userData.fid);
totalMeanAbsDiff = 0;

setOfHYPRFrames = zeros(client_handles.userData.sz,...
    client_handles.userData.sz,...
    client_handles.userData.nTimeFrames);
setOfHYPRFrames_noise = setOfHYPRFrames ;

log_file_initialization_per_algorithm(client_handles.userData.fid);

% make a finite state machine to handle logic
%server1 states
PAUSE=0; PLAY=1; MOVE_TO_NEXT_PROJECTION=2;
MOVE_TO_END_OF_FRAME=3; CANCEL=4;

projectionNumber = 1;
numberOfProjectionsInCurrentFrame = 0;
hyprFrameNumber = 1;
NORMAL  = 1;
ABORTED = 0;

status = NORMAL;

fbp = zeros(client_handles.userData.sz,client_handles.userData.sz);
bp  = zeros(client_handles.userData.sz,client_handles.userData.sz);
fbp_noise = fbp;
bp_noise  = bp;

while projectionNumber<=client_handles.userData.totalNumberOfProjections ...
        && status == NORMAL

    client_handles  = guidata(hObject);
    switch client_handles.interrupt_state.server1_state
        case PAUSE
            set(server_1_handles.statusTag,'String','Click PLAY to continue....');                        
            pause(.1);
        case MOVE_TO_NEXT_PROJECTION
            PARTIAL = false;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'off');

            client_handles.interrupt_state.server1_state = PAUSE;
            guidata(hObject,client_handles);
            make_one_projection();
            projectionNumber = projectionNumber + 1;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'on');

        case MOVE_TO_END_OF_FRAME
            PARTIAL = false;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'off');

            number_of_projections_to_end_of_current_frame = ...
                client_handles.userData.nProjectionsPerFrame-numberOfProjectionsInCurrentFrame;

            for k=1:number_of_projections_to_end_of_current_frame
                make_one_projection();
                projectionNumber = projectionNumber + 1;
            end

            client_handles.interrupt_state.server1_state = PAUSE;
            guidata(hObject,client_handles);
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'on');

        case PLAY
            PARTIAL = true;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'off');
            make_one_projection();
            projectionNumber = projectionNumber + 1;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'on');

        case CANCEL
            status = ABORTED;
            PARTIAL = false;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'off');
    end
end

if status == NORMAL
    set(server_1_handles.statusTag,'String','Completed generating HYPR frames.');
    client_handles.userData.averagedPSNR    = mean(running_psnr_vector);
    client_handles.userData.averagedAbsDiff = totalMeanAbsDiff/client_handles.userData.nTimeFrames;

    client_handles.interrupt_state.server_1_completed_normal = 1;
else
    client_handles.interrupt_state.server_1_completed_normal = 0;
    set(server_1_handles.statusTag,'String','Aborted....');
end
guidata(hObject,client_handles);

end

