%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************
function p = doBackProjection( client_handles , filtered, from, to, MODE, noise_process)

% if noise_process is true, then this function will work on g_noise and
% gc_noise vectors. otherwise, it will work on the g and gc projections.

% g,gc projections are from real data, and g_noise and gc_noise are from
% noise only.

%MODE=0 means from g
%MODE=1 means from gc

%reshuffle to make iradon happy by making angles consecutive
%due to bitreversed order earlier mess which was done to make
%W-HYPR happy

nProjections = to-from+1;
T      = zeros(nProjections,2);
T(:,1) = client_handles.userData.angles(from:to);

T(:,2) = 1:nProjections;
T(:,2) = T(:,2) + (from-1); % adjust to correct sliding window
T      = sortrows(T);

if MODE == false % work on the g or g_noise. False = g, true = gc
    if noise_process
        if filtered
            if nProjections == 1
                p = iradon([client_handles.userData.g_noise(:,T(:,2)) ...
                    client_handles.userData.g_noise(:,T(:,2))],...
                    [T(:,1) T(:,1)],...
                    client_handles.userData.iradon_intepolation,...
                    client_handles.userData.iradon_filter,...
                    client_handles.userData.sz)/2;
            else
                p = iradon(client_handles.userData.g_noise(:,T(:,2)),T(:,1),...
                    client_handles.userData.iradon_intepolation,...
                    client_handles.userData.iradon_filter,...
                    client_handles.userData.sz);
            end
        else
            if nProjections == 1
                p = iradon( [client_handles.userData.g_noise(:,T(:,2)) ...
                    client_handles.userData.g_noise(:,T(:,2))],...
                    [T(:,1) T(:,1)],...
                    'none',...
                    client_handles.userData.sz)/2;
            else
                p = iradon(client_handles.userData.g_noise(:,T(:,2)),T(:,1),'none',client_handles.userData.sz);
            end
        end
    else %not noise process, working on g
        if filtered
            if nProjections == 1
                p = iradon([client_handles.userData.g(:,T(:,2)) ...
                    client_handles.userData.g(:,T(:,2))],...
                    [T(:,1) T(:,1)],...
                    client_handles.userData.iradon_intepolation,...
                    client_handles.userData.iradon_filter,...
                    client_handles.userData.sz)/2;
            else
                p = iradon(client_handles.userData.g(:,T(:,2)),...
                    T(:,1),...
                    client_handles.userData.iradon_intepolation,...
                    client_handles.userData.iradon_filter,...
                    client_handles.userData.sz);
            end
        else
            if nProjections == 1
                p = iradon( [client_handles.userData.g(:,T(:,2))...
                    client_handles.userData.g(:,T(:,2))],...
                    [T(:,1) T(:,1)],...
                    'none',...
                    client_handles.userData.sz)/2;
            else
                p = iradon(client_handles.userData.g(:,T(:,2)),...
                    T(:,1),...
                    'none',...
                    client_handles.userData.sz);
            end
        end
    end
else  % work on the gc or gc_noise
    if noise_process
        if filtered
            if nProjections == 1
                p = iradon([client_handles.userData.gc_noise(:,T(:,2))...
                    client_handles.userData.gc_noise(:,T(:,2))],...
                    [T(:,1) T(:,1)],...
                    client_handles.userData.iradon_intepolation,...
                    client_handles.userData.iradon_filter,...
                    client_handles.userData.sz)/2;
            else
                p = iradon(client_handles.userData.gc_noise(:,T(:,2)),T(:,1),...
                    client_handles.userData.iradon_intepolation,...
                    client_handles.userData.iradon_filter,...
                    client_handles.userData.sz);
            end
        else
            if nProjections == 1
                p = iradon([client_handles.userData.gc_noise(:,T(:,2))...
                    client_handles.userData.gc_noise(:,T(:,2))],...
                    [T(:,1) T(:,1)],...
                    'none',...
                    client_handles.userData.sz)/2;
            else
                p = iradon(client_handles.userData.gc_noise(:,T(:,2)),T(:,1),'none',client_handles.userData.sz);
            end
        end
    else %no noise, work on gc
        if filtered
            if nProjections == 1
                p = iradon([client_handles.userData.gc(:,T(:,2))...
                    client_handles.userData.gc(:,T(:,2))],...
                    [T(:,1) T(:,1)],...
                    client_handles.userData.iradon_intepolation,...
                    client_handles.userData.iradon_filter,...
                    client_handles.userData.sz)/2;
            else
                p = iradon(client_handles.userData.gc(:,T(:,2)),...
                    T(:,1),...
                    client_handles.userData.iradon_intepolation,...
                    client_handles.userData.iradon_filter,...
                    client_handles.userData.sz);
            end
        else
            if nProjections == 1
                p = iradon([client_handles.userData.gc(:,T(:,2))...
                    client_handles.userData.gc(:,T(:,2))],...
                    [T(:,1) T(:,1)],...
                    'none',...
                    client_handles.userData.sz)/2;
            else
                p = iradon(client_handles.userData.gc(:,T(:,2)),...
                    T(:,1),...
                    'none',...
                    client_handles.userData.sz);
            end
        end
    end
end

p(p<0) = 0;
