%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************
%
% script name: example_how_to_generate_mat_file_for_user_projections.m
% Script to generate an example of projection_data.mat for
% testing the user supplied projections, image loading option
% in the HYPR simulator v 1.3
% This options allows the user to load a matlab mat file
% which contains 4 variables. The projections, the angles, the
% number of projection per time frame, and the number of time frames.
%
% (I could ofcourse find number of time frames, given the other 3
% variables, but it is better to let the user supply these so they
% know what they are doing).
%


% example 1
ph           = phantom(256);
nProjections = 128;

angles       = linspace(0,180,nProjections);
projections  = radon(ph,angles);
Np = 8;


save('projection_phantom.mat',...
    'angles',...
    'projections',...
    'Np');

% example 2
nProjections = 64;
angles       = linspace(0,180,nProjections);
projections  = ones(nProjections,nProjections);
Np = 8;

save('projection_data_all_ones.mat',...
    'angles',...
    'projections',...
    'Np');

% example 3 a sin function
nProjections = 64;
angles       = linspace(0,180,nProjections);
projections  = zeros(nProjections,nProjections);
for i=1:nProjections
    projections(:,i) = sind(i*angles);
end
Np = 8;

save('projection_data_sins.mat',...
    'angles',...
    'projections',...
    'Np');

% example 4 a sin*cos function
nProjections = 64;
angles       = linspace(0,180,nProjections);
projections  = zeros(nProjections,nProjections);
for i=1:nProjections
    projections(:,i) = cosd(i*angles).*sind(i*angles);
end
Np           = nProjections;

save('projection_data_cos_sin.mat',...
    'angles',...
    'projections',...
    'Np');
