%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************
function status = generateDataForFig6WrightPaper(hObject)

status = 1;
[client_handles,server_1_handles]=pre_process_data_generation(hObject,0,128,256);

r_small = 6;
r_large = 10;
r_behind = 60;

cx_left  = -15;
cy_left  = 0;

cx_top   = 0;
cy_top   = 15;

cx_right = 15;
cy_right = 0;

cx_down  = 0;
cy_down  = -15;

cx_behind  = 0;
cy_behind  = 0;


gray_level_for_artery = make_gray_level_for_artery(client_handles.userData.totalNumberOfProjections,...
    client_handles.userData.minGrayLevel,...
    client_handles.userData.maxGrayLevel);

gray_level_for_venas = make_gray_level_for_venas(client_handles.userData.totalNumberOfProjections,...
    client_handles.userData.minGrayLevel,...
    client_handles.userData.maxGrayLevel);

gray_level_for_behind = make_gray_level_for_behind(client_handles.userData.totalNumberOfProjections,...
    client_handles.userData.minGrayLevel,...
    client_handles.userData.maxGrayLevel);

currentProjectionNumber = 0;
currentFrameNumber      = 1;
tmp = guidata(hObject);
set(server_1_handles.currentFrameNumberTag,'String',currentFrameNumber);

for i = 1:client_handles.userData.totalNumberOfProjections

    A_left = nma_makeDisk(client_handles.userData.sz ,...
        r_small,...
        cx_left,...
        cy_left,...
        client_handles.userData.minGrayLevel,...
        client_handles.userData.maxGrayLevel);
    A_left(A_left>0) = gray_level_for_artery(i);

    A_top = nma_makeDisk(client_handles.userData.sz ,...
        r_small,...
        cx_top,...
        cy_top,...
        client_handles.userData.minGrayLevel,...
        client_handles.userData.maxGrayLevel);
    A_top(A_top>0) = gray_level_for_artery(i);

    A_right = nma_makeDisk(client_handles.userData.sz ,...
        r_large,...
        cx_right,...
        cy_right,...
        client_handles.userData.minGrayLevel,...
        client_handles.userData.maxGrayLevel);
    A_right(A_right>0) = gray_level_for_venas(i);

    A_down = nma_makeDisk(client_handles.userData.sz ,...
        r_large,...
        cx_down,...
        cy_down,...
        client_handles.userData.minGrayLevel,...
        client_handles.userData.maxGrayLevel);
    A_down(A_down>0) = gray_level_for_venas(i);

    A_behind = nma_makeDisk(client_handles.userData.sz ,...
        r_behind,...
        cx_behind,...
        cy_behind,...
        client_handles.userData.minGrayLevel,...
        client_handles.userData.maxGrayLevel);
    A_behind(A_behind>0) = gray_level_for_behind(i);

    currentProjectionNumber=currentProjectionNumber+1;

    set(server_1_handles.server_1_projection_number_tag,'String',currentProjectionNumber );

    tmp = guidata(hObject);
    if tmp.interrupt_state.server1_state == 4
        status = 0;
        break;
    end

    A = A_left + A_right + A_down + A_top + A_behind;

    client_handles.userData.userImages(:,:,i) =  A;
    [currentFrameNumber,currentProjectionNumber]=...
        data_generation_LOOP_update(client_handles,server_1_handles,A,...
        currentProjectionNumber,currentFrameNumber);
end

if tmp.interrupt_state.server1_state == 4
    return;
end

client_handles.userData.userImagesCreated = 1;
guidata(hObject,client_handles);  %only save images if not aborted

end
%%%%%%%%%%%%%
%
%%%%%%%%%%%%%
function g=make_gray_level_for_artery(nProjections,minI,maxI)
s=floor(nProjections/4);
left_over = nProjections-4*s;

g1=linspace(minI,maxI,s);
g2=linspace(maxI,maxI/2,s);
g3=linspace(maxI/2,maxI/2,2*s+left_over);
g=[g1 g2 g3];
end
%%%%%%%%%%%%%
%
%%%%%%%%%%%%%
function g=make_gray_level_for_venas(nProjections,minI,maxI)
s=floor(nProjections/4);
left_over = nProjections-4*s;

g1=linspace(minI,minI,s);
g2=linspace(minI,maxI,s);
g3=linspace(maxI,maxI/2,s);
g4=linspace(maxI/2,maxI/2,s+left_over);
g=[g1 g2 g3 g4];

end

%%%%%%%%%%%%%
%
%%%%%%%%%%%%%
function g=make_gray_level_for_behind(nProjections,minI,maxI)
%s=floor(nProjections/2);
%left_over = nProjections-2*s;

g=linspace(minI,0.3*maxI,nProjections);
% g2=linspace(0.2*maxI,0.2*maxI,s+left_over);
% g=[g1 g2];

end