%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************
function status = generateDataFor_user_supplied_projections(hObject)

status = 0;
client_handles   = guidata(hObject);
server_1_handles = guidata(client_handles.userData.server_1_handle);


[file_name, path_name] = uigetfile('*.mat', 'Select projection data file');
if( path_name ~= 0 )
    matFileFullPath = [path_name file_name];
    fd = fopen([path_name file_name],'r');
    if(fd == -1)
        uiwait(errordlg(sprintf('Error opening mat file %s for reading. . File could be open by another application.',file_name),'modal'));
        return;
    end
    fclose(fd);
end

data = load(matFileFullPath);

%
% Verify data
%
if ~isfield(data,'angles')
    errordlg(sprintf('mat file %s does not contain [angles] field',file_name),'modal');
    return;
end
if ~isfield(data,'projections')
    uiwait(errordlg(sprintf('mat file %s does not contain [projections] field',file_name),'modal'));
    return;
end
if ~isfield(data,'Np')
    uiwait(errordlg(sprintf('mat file %s does not contain [Np] field',file_name),'modal'));
    return;
end
if (ndims(data.projections)>2)
    uiwait(errordlg(sprintf('mat file %s [projections] has too many dimensions',file_name),'modal'));
    return;
end
[nRow,nCol]=size(data.projections);
if nCol ~= length(data.angles)
    uiwait(errordlg(sprintf('[%s], number of [angles] should be the same as number of columns in [projections]',file_name),'modal'));
    return;
end
if data.Np > length(data.angles)
    errordlg(sprintf('[%s], Np is too large',file_name),'modal');
    return;
end
if mod(length(data.angles),data.Np) ~=0
    errordlg(sprintf('[%s], Np does not divide evenly the number of projections',file_name),'modal');
    return;
end

status = 1;
set(server_1_handles.statusTag,'String','Generating temporal images...');

currentProjectionNumber = 0;
currentFrameNumber      = 1;
set(server_1_handles.currentFrameNumberTag,'String',currentFrameNumber);
client_handles.userData.totalNumberOfProjections = ...
    length(data.angles);

client_handles.userData.nTimeFrames          = length(data.angles)/data.Np;
client_handles.userData.nProjectionsPerFrame = data.Np;

set(server_1_handles.currentFrameNumberTag,'String',currentFrameNumber);

set(client_handles.viewModeGroupTag,'SelectedObject',client_handles.partialViewTag);
set(client_handles.startAngleTag,'String',data.angles(1));
set(client_handles.endingAngleTag,'String',data.angles(end));
set(client_handles.fullViewTag,'Enable','off');
set(client_handles.partialViewTag,'Enable','off');
guidata(hObject,client_handles);

client_handles.userData.angles   = getAngles(hObject,length(data.angles));
client_handles.userData.g        = data.projections;
client_handles.userData.g_length = nRow;

%
% make a quick iradon so we know how much to set the image size to
%
tmp = iradon([data.projections(:,1) data.projections(:,1)],...
    [data.angles(1) data.angles(1)],'none');
client_handles.userData.sz       = size(tmp,1);  %let iradon figure it out

set(client_handles.nTimeFramesTag,'String',client_handles.userData.nTimeFrames);
set(client_handles.nProjectionsTag,'String',client_handles.userData.nProjectionsPerFrame);

guidata(hObject,client_handles);
figure(server_1_handles.figure_server_1);

for i = 1:client_handles.userData.totalNumberOfProjections
    currentProjectionNumber=currentProjectionNumber+1;

    set(server_1_handles.server_1_projection_number_tag,'String',currentProjectionNumber );

    if currentProjectionNumber == client_handles.userData.nProjectionsPerFrame
        currentFrameNumber = currentFrameNumber  + 1;
        currentProjectionNumber = 0;
        if currentFrameNumber<=client_handles.userData.nTimeFrames
            set(server_1_handles.currentFrameNumberTag,'String',...
                sprintf('%d',currentFrameNumber));
        end
    end
    pause(.01);
end

client_handles.userData.userImagesCreated = 1;
guidata(hObject,client_handles);  %only save images if not aborted

end