%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************

% --- Executes on button press in generateHYPRBtnTag.
function generateHYPRBtnTag_Callback(hObject, eventdata, handles)
% hObject    handle to generateHYPRBtnTag (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
%generate all g projections first

is_success = verify_all_UI_input(hObject);
if ~is_success 
    return;
end

PAUSE   = 0;
NORMAL  = 1;
ABORTED = 0;

client_handles   = guidata(hObject);
server_1_handles = guidata(client_handles.userData.server_1_handle);
client_handles.interrupt_state.server1_state     = PAUSE;
client_handles.userData.server1_completed = 0;

guidata(hObject,handles);


msg = sprintf('[%s]%s\n\n\n',datestr(now),'Enter generate HYPR ...');
updateLogFile(msg,client_handles.userData.fid);

%
% Seed the random number generator now. This is done each time
% a new generate HYPR button is pressed. This way we can 
% reproduce any test result each time.
%
seed_RNG(client_handles.userData.fid);

log_all_run_parameter(client_handles);

%make server1 window current
figure(server_1_handles.figure_server_1);

if client_handles.userData.userImagesCreated==0
    set(server_1_handles.statusTag,'String','Please generate user images first...');
    return;
end

resetLastAlgorithmAxes(server_1_handles);
clearAllStatistics(server_1_handles);

set(server_1_handles.statusTag,'String','Reconstruction...generating HYPR frames...');

%turn off the rest and generate HYPR btns untill we are done.
%the generate time frame btn is allready off. User has to hit rest to get
%all back.
set(client_handles.resetBtnTag,'Enable','off');
set(client_handles.generateHYPRBtnTag,'Enable','off');
set(server_1_handles.server_1_generate_HYPR_frames_btn,'Enable','off');

%determine the algorithm to use
if get(client_handles.lrHYPRBtn,'Value')==1
    algHandleTag='lrHYPRBtn';
else
   algHandle    = get(client_handles.selectAlgorithmGroupTag,'SelectedObject');
   algHandleTag = get(algHandle,'Tag');
end

guidata(hObject,client_handles);

%determine iradon parameters
[client_handles.userData.iradon_intepolation,...
    client_handles.userData.iradon_filter] = get_iradon_options(client_handles);

%
%allocate statistics array to keep track of one full HYPR frame
%reconstruction. This is kept in the userData buffer
%
client_handles.userData.stats_array=cell(client_handles.userData.nTimeFrames,1);
guidata(hObject,client_handles);


if ~get(client_handles.client_user_projection_tag,'Value')==1
    %
    % Make the forward projections 'g'. This uses the images that are
    % allready loaded when the user said to generate images and they are
    % saved in the userData off the handles.
    %    
    make_g(hObject);
end

%
% Now that we have the projections created (and saved in the userData
% we can add noise to them
%
% Notice: In the step below, the noise process also starts by making
% g_noise projections as copy of the noise only which is being added to
% the g projections.
%
nma_addNoiseToProjection(hObject);

PARTIAL = false;
set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'on');
set(server_1_handles.server_1_generate_HYPR_frames_btn,'enable','off'); %oh well, fix this

set(server_1_handles.statusTag,'String','Click PLAY to start reconstruction....');

switch algHandleTag
    case {'originalHYPRTag','IHYPRTag'}
        [C,C_noise] = pre_process_step(hObject);
        ITERATE  = false;
        [C,C_noise,status] = originalHYPR(hObject,C,C_noise,ITERATE);
    case {'wrightHuangHYPRTag','IWHYPRTag'}
        [C,C_noise] = pre_process_step(hObject);
        ITERATE  = false;
        [C,C_noise,status] = wrightHuangHYPR(hObject,C,C_noise,ITERATE);
    case {'client_FBP_btn'}
        [C,C_noise,status] = FBP(hObject);
    case {'lrHYPRBtn'}
        [C,C_noise] = pre_process_step(hObject);
        [C,C_noise,status] = HYPR_LR(hObject,C,C_noise);        
end

client_handles = guidata(hObject);

%
% Check if we returned back from a normal return or an aborted return
%
if status == NORMAL
    %
    % Save the above C, which contains now the set of HYPR frames generated
    % so that if we are in iterative mode, we can compare this set with the
    % set from the LAST iteration of I-HYPR. We consider the above run as
    % the initial iteration. And remember to save it back to guidata
    %
    client_handles.userData.hypr_images_first_iteration = C;
    client_handles.userData.hypr_images_last_iteration  = C; %trick, I know.
    client_handles.userData.hypr_images_first_iteration_noise = C_noise;
    client_handles.userData.hypr_images_last_iteration_noise  = C_noise;    
    
    guidata(hObject,client_handles);

    %
    % Notice, the C's below is what was returned back from the
    % calls above to HYPR algorithms, which will be a 3D structure now
    % and not a 2D structure. This is becuase it is now the set of 
    % HYPR images, i.e. a set of images, hence 3D. This is becuase
    % iterative methods use the set of HYPR images as composites. Each 
    % HYPR image is a composite.
    %
    switch algHandleTag
        case {'IWHYPRTag','IHYPRTag'}
            [status,C,C_noise] = iterativeHYPRLoop(hObject,algHandleTag,C,C_noise);

            % Now save the last set of HYPR frames for analysis later
            client_handles.userData.hypr_images_last_iteration = C;
            client_handles.userData.hypr_images_last_iteration_noise = C_noise;

            guidata(hObject,client_handles);
    end
else
    client_handles.userData.hypr_images_first_iteration = [];
    client_handles.userData.hypr_images_last_iteration  = [];
    client_handles.userData.hypr_images_first_iteration_noise = [];
    client_handles.userData.hypr_images_last_iteration_noise  = [];

    guidata(hObject,client_handles);
end

client_handles   = guidata(hObject);
server_1_handles = guidata(client_handles.userData.server_1_handle);

set(client_handles.resetBtnTag,'Enable','on');
set(client_handles.generateHYPRBtnTag,'Enable','on');
set(server_1_handles.server_1_generate_HYPR_frames_btn,'Enable','on');

client_handles.userData.server1_completed = 1;

PARTIAL = false;
set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'off');

guidata(hObject,client_handles);

if status == ABORTED
    set(server_1_handles.statusTag,'String','Aborted....');
    server_2_handles = guidata(client_handles.userData.server_2_handle);

    set(server_2_handles.server_2_pause_btn,'Enable','off');
    set(server_2_handles.server_2_play_btn,'Enable','off');
    set(server_2_handles.server_2_move_one_frame_btn,'Enable','off');
    set(server_2_handles.server_2_stop_btn,'Enable','off');

    figure(client_handles.figure1);
else
    %so that non-iterative can also playback, set this below on 'on' anyway
    set(client_handles.playback_btn,'Enable','on'); 
    set(server_1_handles.statusTag,'String','Completed HYPR reconstruction....');
end

end

%%%%%%%%%%%
% This function makes the Composite the gc projections from the
% composite, and also makes the noise composite and the gc_noise, which
% are the projections from the noise composite
%%%%%%%%%%%
function [C,C_noise] = pre_process_step(hObject)

client_handles   = guidata(hObject);
server_1_handles = guidata(client_handles.userData.server_1_handle);

centered_frame_number = 1;
[C,C_noise] = get_composite_image(client_handles,centered_frame_number);
                                     
display_image_with_correct_gray_scale(server_1_handles.server_1_C_Axes,...
    'CImageAxes',...
    C,...
    client_handles,...
    server_1_handles);

guidata(hObject,client_handles);

%
% Now generate the forward projectins from the composite. the gc's
%to collect gc projections in

gc_length = length(radon(C,client_handles.userData.angles(1)));

client_handles.userData.gc = zeros(gc_length,...
    client_handles.userData.totalNumberOfProjections);

client_handles.userData.gc_noise = zeros(gc_length,...
    client_handles.userData.totalNumberOfProjections);

for i = 1:client_handles.userData.totalNumberOfProjections
    client_handles.userData.gc(:,i) = radon(C,client_handles.userData.angles(i)); %take projection at bitreversed order angle
    client_handles.userData.gc_noise(:,i) = radon(C_noise,client_handles.userData.angles(i)); %take projection at bitreversed order angle
end

guidata(hObject,client_handles);

end
