%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************
function generateTemporalImagesTag_Callback(hObject, eventdata, client_handles)

% --- Executes on button press in generateTemporalImagesTag.
% hObject    handle to generateTemporalImagesTag (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

is_success = verify_all_UI_input(hObject);
if ~is_success 
    return;
end


if client_handles.userData.userImagesCreated==1
    return;
end

resetLastAlgorithmAxes(guidata(client_handles.userData.server_1_handle));

set_image_selection_buttons(client_handles,'off');
set(client_handles.generateTemporalImagesTag,'Enable','off');
set(client_handles.resetBtnTag,'Enable','off');
set(client_handles.generateHYPRBtnTag,'Enable','off');
set(client_handles.playback_btn,'Enable','off');

%disable these since they affect image generation only and
%we don't want user mucking these up and complain why it is ain't working
%as expected. use RESET button to enable them all again

server_1_handles = guidata(client_handles.userData.server_1_handle);
PARTIAL=true;
set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'off')

clearAllStatistics(server_1_handles);
client_handles.interrupt_state.server1_state = 1;
guidata(hObject,client_handles);

%decide on which image to work with
h              = get(client_handles.selectSourceImageTag);
hSelectedImage = h.SelectedObject;

set(server_1_handles.currentFrameNumberTag,'String','');
set(server_1_handles.statusTag,'String','Starting MRI...');

tag = get(hSelectedImage,'Tag');
switch  tag  % Get Tag of selected object
    case 'wrightPaperDiskTag'
        status = generateDataForWrightHuang(hObject);
    case 'movingDiskUpDownTag'
        status = generateDateForUpDown(hObject);
    case 'movingDiskDiagonalTag'
        status = generateDateForDiagonal(hObject);
    case 'monkeyTag'
        status = generateDataForMonkey(hObject);
    case 'twoDisksApartTag'
        status = generateDataFortwoDisksApart(hObject);
    case 'twoDisksCloseTag'
        status = generateDataFortwoDisksClose(hObject);
    case 'timeInvarientDiskTag'
        status = generateDataForTimeInvarientDisk(hObject);
    case 'phantomBtn'
        status = generateDataForPhantom(hObject);
    case 'twoDisksDifferentIntensityTag'
        status = generateDataForTwoDisksDifferentIntensity(hObject);
    case 'diskWithHoleTag'
        status = generateDataForDiskWithHoleTag(hObject);
    case 'lowerLegTag'
        status = generateDataForlowerLeg(hObject);
    case 'paperclipPhantomTag'
        status = generateDataForpaperclipPhantom(hObject);
    case 'Fig2WrightPaperTag'
        status = generateDataForFig2WrightPaper(hObject);
    case 'Fig3WrightPaperTag'
        status = generateDataForFig3WrightPaper(hObject);
    case 'Fig5WrightPaperTag'
        status = generateDataForFig5WrightPaper(hObject);
    case 'Fig6WrightPaperTag'
        status = generateDataForFig6WrightPaper(hObject);
    case 'Fig4IHYPRPaperTag'
        status = generateDataForFig4IHYPRPaper(hObject);
    case 'Fig2IHYPRPaperTag'
        status = generateDataForFig2IHYPRPaper(hObject);
    case 'deltaPointSourceTag'
        status = generateDataForPointImpulse(hObject);
    case 'lineImpulseTag'
        status = generateDataForLineImpulse(hObject);
    case 'client_all_black_disk_tag'
        status = generateDataFor_all_black_disk(hObject);
    case 'client_user_projection_tag'
        status = generateDataFor_user_supplied_projections(hObject);
end

if status == 1
    updateLogFile(sprintf('generated data for simulation [%s]\n',tag),...
        client_handles.userData.fid);

    nTimeFrames = str2double(get(client_handles.nTimeFramesTag,'String'));
    nProjectionsPerFrame = str2double(get(client_handles.nProjectionsTag,'String'));

    msg = [sprintf('%s\n','Completed image generation...'),...
        sprintf('number of time frames = [%d]\n',nTimeFrames),...
        sprintf('number of projections per time frame = [%d]\n',nProjectionsPerFrame)];
    updateLogFile(msg,client_handles.userData.fid);
    set(server_1_handles.statusTag,'String','Completed MRI...Click Generate HYPR');
    set(client_handles.generateHYPRBtnTag,'Enable','on');
    set(server_1_handles.server_1_generate_HYPR_frames_btn,'Enable','on');
else
    updateLogFile(sprintf('Aborted simulation for [%s]\n',tag),client_handles.userData.fid);
    set(server_1_handles.statusTag,'String','Aborted MRI by request...');
    %since aborted, keep data generation button on
    set(client_handles.generateTemporalImagesTag,'Enable','on');
    set_image_selection_buttons(client_handles,'on');
    set(client_handles.generateHYPRBtnTag,'Enable','off');
    figure(client_handles.figure1);
end

set(client_handles.resetBtnTag,'Enable','on');

end
