%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************

function [currentHYPRframe,...
    running_psnr_vector,...
    totalMeanAbsDiff] = generate_FBP_HYPR_frame(...
    fbp,...
    client_handles,...
    server_1_handles,...
    hyprFrameNumber,...
    running_psnr_vector,...
    totalMeanAbsDiff,...
    numberOfCurrentProjection)


set(server_1_handles.currentFrameNumberTag,'String',...
    sprintf('%d',hyprFrameNumber));

%Make HYPRframe
currentHYPRframe = fbp;

%display its mean intensity
%now center it around the time frame window
plot(server_1_handles.meanIntensityAxes,...
    (hyprFrameNumber-1)*client_handles.userData.nProjectionsPerFrame+...
    round(client_handles.userData.nProjectionsPerFrame/2),...
    mean(currentHYPRframe(:)),'ro');

if client_handles.userData.totalNumberOfProjections==1
    xlim(server_1_handles.meanIntensityAxes,[1,2]);
else
    xlim(server_1_handles.meanIntensityAxes,[1,client_handles.userData.totalNumberOfProjections]);
end

%display its max intensity
axes(server_1_handles.maxIntensityAxes);
%now center it around the time frame window
plot(server_1_handles.maxIntensityAxes,...
    (hyprFrameNumber-1)*client_handles.userData.nProjectionsPerFrame+...
    round(client_handles.userData.nProjectionsPerFrame/2),...
    max(currentHYPRframe(:)),'ro');
if client_handles.userData.totalNumberOfProjections==1
    xlim(server_1_handles.maxIntensityAxes,[1,2]);
else
    xlim(server_1_handles.maxIntensityAxes,[1,client_handles.userData.totalNumberOfProjections]);
end

%save a copy of HYPR frame image on disk
s           = which('nma_HYPR');
s           = s(1:end-10);
logFileName = [s sprintf('hyperframe_%d.png',hyprFrameNumber)];
imwrite(uint8(currentHYPRframe),logFileName,'png');

% DISPLAY current HYPR frame, and also display the time frame as
% well to make it easier to compare
display_image_with_correct_gray_scale(...
    server_1_handles.server_1_HYPR_Axes,...
    'currentHYPRFrame2Axes',...
    currentHYPRframe,...
    client_handles,server_1_handles);

title(server_1_handles.server_1_HYPR_Axes,'FBP image');

%find RMSE and update the screen statistics
if ~get(client_handles.client_user_projection_tag,'Value')==1
    running_psnr_vector = nma_update_PSNR(currentHYPRframe,...
        hyprFrameNumber,client_handles,server_1_handles,running_psnr_vector);
    set(server_1_handles.PSNRTag,'String',sprintf('%6.3f',running_psnr_vector(hyprFrameNumber)));

    %find current mean of errors over all frames seen so far
    set(server_1_handles.PSNRMeanTag,'String',...
        sprintf('%6.2f',mean(running_psnr_vector(1:hyprFrameNumber))));

    %update the ABS difference plot
    userImages = client_handles.userData.userImages;
    averageImage = sum( userImages(:,:,numberOfCurrentProjection-...
        client_handles.userData.nProjectionsPerFrame+1:numberOfCurrentProjection),3 )...
        / client_handles.userData.nProjectionsPerFrame;
    absDiff = imabsdiff(currentHYPRframe,averageImage);
    currentHYPRFrameMeanAbsDiff = mean(absDiff(:));
    set(server_1_handles.currentFrameMeanAbsErrorTag,'String',sprintf('%6.2f',currentHYPRFrameMeanAbsDiff));
    totalMeanAbsDiff = totalMeanAbsDiff + currentHYPRFrameMeanAbsDiff;
    meanTotalMeanAbsDiff = totalMeanAbsDiff/hyprFrameNumber;
    set(server_1_handles.runningMeanAbsDiffTag,'String',sprintf('%6.2f',meanTotalMeanAbsDiff));
end

if get(server_1_handles.abs_error_server1_btn,'Value')==1
    display_image_with_correct_gray_scale(server_1_handles.error_server1_panel_tag,...
        'histogramAxes',...
        absDiff,...
        client_handles,...
        server_1_handles);
else
    if ~get(client_handles.client_user_projection_tag,'Value')==1
        if get(server_1_handles.server_1_horiz_section_view,'Value')==1
            nCols       = size(currentHYPRframe,1);
            middle_row  = round(nCols/2);
            HYPR_strip  = currentHYPRframe(middle_row,:);
            image_strip = averageImage(middle_row,:);

            y_max = max( [max(HYPR_strip(:)) max(image_strip(:))] );
            h = subplot(1,1,1,'Parent',server_1_handles.error_server1_panel_tag);
            plot(1:nCols,HYPR_strip(:),'k',1:nCols,image_strip(:),'r');
            if y_max>0
                set(h,'Ylim',[0 y_max+0.1*y_max]);
            end
            legend('hypr','image');
        else
            y_max = max( [ max(averageImage(:)) max(currentHYPRframe(:))]);
            y_max = y_max+.05*y_max;

            h1 = subplot(2,1,1,'Parent',server_1_handles.error_server1_panel_tag);
            plot(averageImage(:));
            set(h1,'FontSize',7);
            set(h1,'Ylim',[0 y_max]);
            set(h1,'XTick',[]);
            title(h1,'time frame');

            h1 = subplot(2,1,2,'Parent',server_1_handles.error_server1_panel_tag);
            plot(currentHYPRframe(:),'r.');
            set(h1,'FontSize',7);
            set(h1,'Ylim',[0 y_max]);
            set(h1,'XTick',[]);
            title(h1,'HYPR frame');
        end
    end
end
