%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************
function [status,hypr_images_last_iteration,hypr_images_last_iteration_noise] = ...
    iterativeHYPRLoop(hObject,tag,C,C_noise)

%for status
NORMAL  = 1;
ABORTED = 0;

client_handles   = guidata(hObject);
server_1_handles = guidata(client_handles.userData.server_1_handle);

nIterationsForIHYPRTag = str2double(get(client_handles.nIterationsForIHYPRTag,'String'));

msg = sprintf('Enter %s, number of iterations to do %d\n',tag,nIterationsForIHYPRTag);
updateLogFile(msg,client_handles.userData.fid);

PSNRHistory = zeros(nIterationsForIHYPRTag+1,1);

%this is the main iterative HYPR loop
status = NORMAL;

for i = 1:nIterationsForIHYPRTag
    set(server_1_handles.statusTag,'String',sprintf('Starting iterative HYPR step [%d]',i));

    cla(server_1_handles.meanIntensityAxes,'reset');
    cla(server_1_handles.maxIntensityAxes,'reset');
    cla(server_1_handles.PSNRAxes,'reset');
    %    cla(server_1_handles.intensityAxes,'reset');
    %    cla(server_1_handles.temporalProfileAxes,'reset');

    PSNRHistory = update_iterative_HYPR_error_plot(server_1_handles,...
        client_handles,PSNRHistory,nIterationsForIHYPRTag,i);
    %guidata(hObject,handles);

    ITERATE=true;
    switch tag
        case 'IHYPRTag'
            [C,C_noise,status] = originalHYPR(hObject,C,C_noise,ITERATE);
        case 'IWHYPRTag'
            [C,C_noise,status] = wrightHuangHYPR(hObject,C,C_noise,ITERATE);
    end

    client_handles   = guidata(hObject);
    server_1_handles = guidata(client_handles.userData.server_1_handle);

    msg = sprintf('I-HYPR completed step %d\n',i);
    updateLogFile(msg,client_handles.userData.fid);

    if status == ABORTED
        break;
    end
end

client_handles   = guidata(hObject);
server_1_handles = guidata(client_handles.userData.server_1_handle);

if status == NORMAL
    PSNRHistory=update_iterative_HYPR_error_plot(server_1_handles,...
        client_handles,PSNRHistory,...
        nIterationsForIHYPRTag,nIterationsForIHYPRTag+1);

    set(server_1_handles.statusTag,'String',sprintf('Completed iterative HYPR [%d] steps',...
        nIterationsForIHYPRTag));

    %log the PSNR history
    updateLogFile(sprintf('Final table of average relative RMSE per full HYPR iteration\n'),...
        client_handles.userData.fid);
    updateLogFile(sprintf('iteration\tmean_RMSE\n'),client_handles.userData.fid);
    for i=1:length(PSNRHistory)
        updateLogFile(sprintf('%d\t%f\n',i,PSNRHistory(i)),client_handles.userData.fid);
    end

    hypr_images_last_iteration       = C;
    hypr_images_last_iteration_noise = C_noise;
    set(client_handles.playback_btn,'Enable','on');
else
    hypr_images_last_iteration       = [];
    hypr_images_last_iteration_noise = [];
    updateLogFile('Aborted Iterative HYPR....\n',client_handles.userData.fid);
end
%guidata(hObject,handles);

%%%%%%%%%%%%%
%
%
%%%%%%%%%%%%%%
function PSNRHistory=update_iterative_HYPR_error_plot(server_1_handles,...
    client_handles,PSNRHistory,nIterationsForIHYPRTag,IHYPR_iteration_number)

axes(server_1_handles.IHYPRErrorAxes);
PSNRHistory(IHYPR_iteration_number) = client_handles.userData.averagedPSNR;
plot(1:IHYPR_iteration_number,PSNRHistory(1:IHYPR_iteration_number),'.');
hold(server_1_handles.IHYPRErrorAxes,'on');
plot(1:IHYPR_iteration_number,PSNRHistory(1:IHYPR_iteration_number));
title(server_1_handles.IHYPRErrorAxes,'relative RMSE per HYPR iteration','FontSize', 8); 
ylabel(server_1_handles.IHYPRErrorAxes,'% RMSE');
%xlabel(handles.IHYPRErrorAxes,'iteration');
xlim(server_1_handles.IHYPRErrorAxes,[0 nIterationsForIHYPRTag+1]);

