%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************
function log_all_run_parameter(client_handles)
%function log_all_run_parameter()
%
%This function is called when user hits the RUN command
%it sends to the log file all the run parameters

fid = client_handles.userData.fid;
updateLogFile(sprintf('\n\n>>>>>>>>> NEW RUN STARTING\n'),fid);

%
% do the algorithm type
%
if get(client_handles.lrHYPRBtn,'Value')==1
    algHandleTag = 'HYPR-LR';
else
    algHandle    = get(client_handles.selectAlgorithmGroupTag,'SelectedObject');
    algHandleTag = get(algHandle,'Tag');
end

updateLogFile(sprintf('Algorithm: %s\n',algHandleTag),fid);

%
% do the object type
%
h    = get(client_handles.selectSourceImageTag,'SelectedObject');
tag  = get(h,'Tag');

updateLogFile(sprintf('user image is [%s]\n',tag),fid);
if strcmp(tag,'deltaPointSourceTag')==1
    r = str2double(get(client_handles.client_point_impulse_radius_tag,'String'));
    updateLogFile(sprintf('Point impulse radius [%3.2f] pixels\n',r),fid);
end

%
% do the noise
%
if get(client_handles.addNoiseTag,'Value')==1
    updateLogFile(sprintf('NOISE is being added\n'),fid);
    if get(client_handles.poissonBtnTag,'Value')==1
        lambda = str2double(get(client_handles.lambdaValueTag,'String'));
        updateLogFile(sprintf('Using Poisson noise, lambda=%f. NOT adding in quadrature\n',lambda),fid);
    elseif get(client_handles.normalBtnTag,'Value')==1
        theMean = str2double(get(client_handles.normalMeanTag,'String'));
        sd      = str2double(get(client_handles.normalSDTag,'String'));
        updateLogFile(sprintf('Using Gaussian noise, mean=%f, SD=%f percentage. Adding in quadrature\n',...
            theMean,sd),fid);
        max_g = max(client_handles.userData.g(:));
        sd = sd/100*max_g;
        updateLogFile(sprintf('Maximum porjection signal found to be [%f], hence absolute value of std is calculated to be [%f]\n',...
            max_g,sd),fid);
    elseif get(client_handles.uniformNoiseBtn,'Value')==1
        theMin = str2double(get(client_handles.minUniformTag,'String'));
        theMax = str2double(get(client_handles.maxUniformTag,'String'));

        updateLogFile(sprintf('Using uniform noise, min=%f, max=%f. NOT adding in quadrature\n',...
            theMin,theMax),fid);
    end
else
    updateLogFile(sprintf('No NOISE is being added\n'),client_handles.userData.fid);
end

%
% do the iradon parameters
%
updateLogFile(sprintf('iradon parameters: interpolation [%s], filter [%s]\n',...
    client_handles.userData.iradon_intepolation,...
    client_handles.userData.iradon_filter),...
    fid);
%
% do the view
%
viewHandle    = get(client_handles.viewModeGroupTag,'SelectedObject');
viewHandleTag = get(viewHandle,'Tag');

if strcmp(viewHandleTag,'fullViewTag')
    updateLogFile(sprintf('Using Full view: 0 to 180 degrees\n'),fid);
else
    startAngle = str2double(get(client_handles.startAngleTag,'String'));
    endAngle   = str2double(get(client_handles.endingAngleTag,'String'));

    updateLogFile(sprintf('Using partial view: %2.2f to %2.2f degrees\n',...
        startAngle, endAngle),fid);
end

%
% do Np stuff
%
updateLogFile(sprintf('Number of time frames [%d]\n',...
    client_handles.userData.nTimeFrames),fid);
updateLogFile(sprintf('Number of projections per time frame [%d]\n',...
    client_handles.userData.nProjectionsPerFrame),fid);
updateLogFile(sprintf('Total of projections [%d]\n',...
    client_handles.userData.totalNumberOfProjections),fid);
updateLogFile(sprintf('Image size = %d x %d pixels\n',...
    client_handles.userData.sz,client_handles.userData.sz),fid);
updateLogFile(sprintf('Image gray level. Minumum %2.3f, Maximum %2.3f\n',...
    client_handles.userData.minGrayLevel,client_handles.userData.maxGrayLevel),fid);

updateLogFile(sprintf('projection length = %d pixels\n',...
    client_handles.userData.g_length),fid);

%
% do how we handle p/pc
%
ratio_handle = get(client_handles.client_handling_zero_P_over_PC_group_tag,'SelectedObject');
option_tag   = get(ratio_handle,'Tag');

switch option_tag
    case 'client_set_pc_zero_btn'
        updateLogFile(sprintf('Handling P/Pc ratio when Pc is very small: Setting composite pixel to zero\n'),fid);
    otherwise
        updateLogFile(sprintf('Handling P/Pc ratio when Pc is very small: Using threshold level [%f]\n',...
            str2double(get(client_handles.client_threshold_tag,'String'))),fid);
end

%
% do HYPR-LR parameters
%
if get(client_handles.lrHYPRBtn,'Value')==1
    if get(client_handles.client_LR_average_tag,'Value')==1
        order_of_filter = get(client_handles.client_LR_average_tag,'Value');
        updateLogFile(sprintf('HYPR-LR using Averaging filter of order [%d]\n',...
            order_of_filter+2),fid);

    elseif get(client_handles.client_LR_gaussian_tag,'Value')==1
        order_of_filter =  get(client_handles.client_filter_LR_order_gaussian_tag,'Value');
        sigma = str2double(get(client_handles.client_filter_LR_sigma_gaussian_tag,'String'));
        updateLogFile(sprintf('HYPR-LR using Gaussian of order [%d] and std[%f]\n',...
            order_of_filter+2,sigma),fid);

    elseif  get(client_handles.client_LR_circle_tag,'Value')==1
        radius_of_filter = str2double(get(client_handles.client_filter_LR_r_circle_tag,'String'));
        updateLogFile(sprintf('HYPR-LR using circular Averaging filter of radius [%4.1f]\n',...
            radius_of_filter),fid);

    elseif get(client_handles.client_LR_median_tag,'Value')==1
        order_of_filter = get(client_handles.client_filter_LR_order_median_tag,'Value');
        updateLogFile(sprintf('HYPR-LR using median filter of order [%d]\n',...
            order_of_filter+2),fid);
    else %wiener
        order_of_filter = get(client_handles.client_filter_order_wiener_tag,'Value');
        updateLogFile(sprintf('HYPR-LR using wiener filter of order [%d]\n',...
            order_of_filter+2),fid);
    end
end

%
% do the Sliding window stuff
%
if get(client_handles.client_sliding_window_tag,'Value')==1
    window_size = get(client_handles.client_sliding_window_size_tag,'Value');
    updateLogFile(sprintf('Using sliding window for composite. Size is [%d]\n',...
        window_size*2-1),fid);
else
    updateLogFile(sprintf('NOt using sliding window for composite\n'),fid);
end