%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************
function     nma_addNoiseToProjection(hObject)
%function     nma_addNoiseToProjection(hObject)
%
% Adds noise to a projection

handles  = guidata(hObject);

if get(handles.addNoiseTag,'Value')==1
    nma_addNoiseToProjectionLevel2(hObject,0);
else
    nma_addNoiseToProjectionLevel2(hObject,1);
end
%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%
function  nma_addNoiseToProjectionLevel2(hObject,skipNoise)

handles  = guidata(hObject);

handles.userData.mean_noise_per_projection = zeros(handles.userData.totalNumberOfProjections,1);
handles.userData.g_noise = zeros(...
    handles.userData.g_length,...
    handles.userData.totalNumberOfProjections);

guidata(hObject,handles); %save it before I forget
handles  = guidata(hObject);

if skipNoise
    NoNoiseVector = zeros(handles.userData.g_length,1);
    add_in_quadrature = false;

    for i = 1:handles.userData.totalNumberOfProjections

        handles.userData.g(:,i) = addNoise(handles,...
            i,...
            handles.userData.g(:,i), ...
            NoNoiseVector, ...
            handles.userData.angles(i),...
            add_in_quadrature);

        handles.userData.mean_noise_per_projection(i) = 0;
    end
else
    %find which distribution
    if get(handles.poissonBtnTag,'Value')==1

        lambda = str2double(get(handles.lambdaValueTag,'String'));
        add_in_quadrature = false;

        for i = 1:handles.userData.totalNumberOfProjections
            handles.userData.g_noise(:,i) = poissrnd(lambda,handles.userData.g_length,1);

            handles.userData.g(:,i) = addNoise(...
                handles,...
                i, ...
                handles.userData.g(:,i), ...
                handles.userData.g_noise(:,i),...
                handles.userData.angles(i),...
                add_in_quadrature);

            handles.userData.mean_noise_per_projection(i) = mean(handles.userData.g_noise(:,i));
        end
    elseif get(handles.normalBtnTag,'Value')==1

        theMean = str2double(get(handles.normalMeanTag,'String'));
        sd      = str2double(get(handles.normalSDTag,'String'));

        add_in_quadrature = true;

        %
        % The std read from the UI is a precentage, not an absolute value.
        % It is a precentage of the maximum value of the projections. i.e.
        % We look at all the projections, all of them, and pick the max
        % value found. Then the std is a percentage of that max found.
        %
        % adjusted on 071808

        max_g = max(handles.userData.g(:));

        for i = 1:handles.userData.totalNumberOfProjections
            handles.userData.g_noise(:,i) = theMean+sd/100*max_g*...
                randn(length(handles.userData.g(:,i)),1);

            handles.userData.g(:,i) = addNoise(...
                handles,...
                i,...
                handles.userData.g(:,i),...
                handles.userData.g_noise(:,i), ...
                handles.userData.angles(i),...
                add_in_quadrature);

            handles.userData.mean_noise_per_projection(i) = mean(handles.userData.g_noise(:,i));
        end
    elseif get(handles.uniformNoiseBtn,'Value')==1
        theMin = str2double(get(handles.minUniformTag,'String'));
        theMax = str2double(get(handles.maxUniformTag,'String'));

        add_in_quadrature = false;

        for i = 1:handles.userData.totalNumberOfProjections
            handles.userData.g_noise(:,i) = theMin + (theMax-theMin ).*rand(handles.userData.g_length,1);
            handles.userData.g(:,i) = addNoise(handles,...
                i,...
                handles.userData.g(:,i) , ...
                noise_vector,...
                handles.userData.angles(i),add_in_quadrature);

            handles.userData.mean_noise_per_projection(i) = mean(handles.userData.g_noise(:,i));
        end
    end
end

guidata(hObject,handles);

%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%
function  g = addNoise(handles,projectionNumber, g,...
    noise,angle,add_in_quadrature)

g_before_noise = g;
if add_in_quadrature
    g = sqrt(g_before_noise.^2 + noise.^2);
else
    g = g + noise;
end

server_1_handles = guidata(handles.userData.server_1_handle);

if get(server_1_handles.turnOnNoiseUpdatesTag,'Value')==1
    axes(server_1_handles.noiseEffectAxes);
    plot(g_before_noise);
    hold(server_1_handles.noiseEffectAxes,'on');
    plot(g,'r');
    title(sprintf('proj.[%d], angle [%4.2f]',projectionNumber,angle),'FontSize', 8);
    hold(server_1_handles.noiseEffectAxes,'off');
    axes(server_1_handles.noiseAxes);
    plot(noise);
    title('noise vector', 'FontSize', 8);    
end
