%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************
function [currentHYPRframe,...
    running_psnr_vector,...
    totalMeanAbsDiff] = nma_generate_HYPR_frame(...
    hObject,...
    hyprFrameNumber,...
    C,...
    iterativeMode,...
    mask,...
    running_psnr_vector,...
    totalMeanAbsDiff,...
    numberOfCurrentProjection,...
    current_p,...
    current_pc)


%====================
    function update_special_for_WH_disk()
        h              = get(client_handles.selectSourceImageTag);
        hSelectedImage = h.SelectedObject;
        switch get(hSelectedImage,'Tag')   % Get Tag of selected object
            case 'wrightPaperDiskTag'
                %update temporal profile
                middleRow = round(client_handles.userData.sz/2);
                hyprFrameIntensity(hyprFrameNumber) = sum(currentHYPRframe(middleRow,:))/50;  %fix me
                hyprFrameProjectionNumbers(hyprFrameNumber) = hyprFrameNumber*client_handles.userData.nProjectionsPerFrame;

                hold(server_1_handles.temporalProfileAxes,'on');
                axes(server_1_handles.temporalProfileAxes);
                plot(hyprFrameProjectionNumbers(1:hyprFrameNumber) ,...
                    hyprFrameIntensity(1:hyprFrameNumber),'.');
                xlim([0 client_handles.userData.totalNumberOfProjections]);
                ylim([client_handles.userData.minGrayLevel client_handles.userData.maxGrayLevel]);

                %generate spatial profile
                profileOriginal     = sum(userImages(middleRow,1:end,...
                    numberOfCurrentProjection-client_handles.userData.nProjectionsPerFrame+1:numberOfCurrentProjection),3);
                meanProfileOriginal = profileOriginal/client_handles.userData.nProjectionsPerFrame;

                axes(server_1_handles.intensityAxes);
                plot(0:client_handles.userData.sz-1, meanProfileOriginal);
                ylim([client_handles.userData.minGrayLevel client_handles.userData.maxGrayLevel+client_handles.userData.maxGrayLevel*.2]);
                xlim([0 client_handles.userData.sz-1]);
                hold(server_1_handles.intensityAxes,'on');
                plot(0:client_handles.userData.sz-1,currentHYPRframe(middleRow,:),'k' );
                plot(0:client_handles.userData.sz-1,currentC(middleRow,:),'r' );
                hold(server_1_handles.intensityAxes,'off');
        end

    end

%====================
    function SNR_hypr_paper_calculations()
        N_r   = length(client_handles.userData.g(:,1));  % number readout points
        N_pix = size(currentC,1);   %number of pixels
        N_p   = client_handles.userData.nProjectionsPerFrame;
        N_f   = client_handles.userData.nTimeFrames;
        f     = str2double(get(client_handles.client_vascular_sparsity_factor_tag,'String'));
        N_v   = N_pix * f;

        % Now find sigma

        sigma = sqrt(sum(client_handles.userData.mean_noise_per_projection...
            (numberOfCurrentProjection:-1:numberOfCurrentProjection-...
            client_handles.userData.nProjectionsPerFrame+1).^2));
        sigma = sigma/client_handles.userData.nProjectionsPerFrame;

        %
        % Now compute equation A(5) in paper
        %
        if abs(sigma) > 2*eps
            SNR_current_hypr = C * sqrt(N_r * N_p * N_f)/...
                (sigma *  sqrt( 1 +  N_f/N_v^2 + N_pix/(N_p*N_v^2)) );
        else
            SNR_current_hypr = inf;
        end

        set(server_1_handles.SNR_current_frame_tag,'String',mean(SNR_current_hypr(:)));

        %
        % Now find SNR of composite
        %

        SNR_current_composite = SNR_current_hypr*sqrt(1+ N_f/N_v^2 + N_pix/(N_p*N_v^2));
        set(server_1_handles.SNR_current_composite_tag,'String',mean(SNR_current_composite(:)));
    end


client_handles   = guidata(hObject);
server_1_handles = guidata(client_handles.userData.server_1_handle);
client_handles.userData.stats_struct.frame = hyprFrameNumber;
guidata(hObject,client_handles);

if iterativeMode
    currentC = C(:,:, hyprFrameNumber);
else
    currentC = C;
end

%
% display C, the composite
%
display_image_with_correct_gray_scale(server_1_handles.server_1_C_Axes,...
    'CImageAxes',currentC,client_handles,server_1_handles);
title(sprintf('C, mean=%6.2f, max=%6.2f',...
    mean(currentC(:)),max(currentC(:))),'FontSize',7);
%
% MASK
%
if get(server_1_handles.turnOnProjectionUpdatesTag,'Value')==1
    plot_the_mask(server_1_handles,client_handles,mask)
end

%
% Make HYPRframe and display it and save copy to disk
% and update its statistics
%
currentHYPRframe = mask .* currentC;
display_image_with_correct_gray_scale(server_1_handles.server_1_HYPR_Axes,...
    'currentHYPRFrame2Axes',currentHYPRframe,client_handles,server_1_handles)
s           = which('nma_HYPR');
s           = s(1:end-10);
logFileName = [s sprintf('hyperframe_%d.png',hyprFrameNumber)];
imwrite(uint8(currentHYPRframe),logFileName,'png');

update_image_statistics('HYPR',...
    currentHYPRframe,...
    hObject);
client_handles =   guidata(hObject);

%
%display its mean intensity and center it around the time frame window
%
axes(server_1_handles.meanIntensityAxes);
plot((hyprFrameNumber-1)*client_handles.userData.nProjectionsPerFrame+...
    round(client_handles.userData.nProjectionsPerFrame/2),...
    mean(currentHYPRframe(:)),'ro');
if client_handles.userData.totalNumberOfProjections==1
    xlim(server_1_handles.meanIntensityAxes,[1,2]);
else
    xlim(server_1_handles.meanIntensityAxes,[1,client_handles.userData.totalNumberOfProjections]);
end

%
% display its max intensity and center it around the time frame window
%
axes(server_1_handles.maxIntensityAxes);
plot((hyprFrameNumber-1)*client_handles.userData.nProjectionsPerFrame+...
    round(client_handles.userData.nProjectionsPerFrame/2),...
    max(currentHYPRframe(:)),'ro');
if client_handles.userData.totalNumberOfProjections==1
    xlim(server_1_handles.maxIntensityAxes,[1,2]);
else
    xlim(server_1_handles.maxIntensityAxes,[1,client_handles.userData.totalNumberOfProjections]);
end

%
% find RMSE and update the screen statistics
%
if ~get(client_handles.client_user_projection_tag,'Value')==1
    running_psnr_vector = nma_update_PSNR(currentHYPRframe,...
        hyprFrameNumber,client_handles,server_1_handles,running_psnr_vector);
    set(server_1_handles.PSNRTag,'String',sprintf('%6.3f',running_psnr_vector(hyprFrameNumber)));

    client_handles.userData.stats_struct.RMSE = running_psnr_vector(hyprFrameNumber);
else
    client_handles.userData.stats_struct.RMSE = 0;
end
%
% find current mean of errors over all frames seen so far
%
set(server_1_handles.PSNRMeanTag,'String',...
    sprintf('%6.2f',mean(running_psnr_vector(1:hyprFrameNumber))));

%
%update the ABS difference plot
%
if ~get(client_handles.client_user_projection_tag,'Value')==1
    userImages = client_handles.userData.userImages;
    averageImage = sum( userImages(:,:,numberOfCurrentProjection-...
        client_handles.userData.nProjectionsPerFrame+1:numberOfCurrentProjection),3 )...
        / client_handles.userData.nProjectionsPerFrame;

    guidata(hObject,client_handles);
    update_image_statistics('MEAN_TRUE_IMAGE',...
        averageImage,...
        hObject);
    client_handles =   guidata(hObject);

    absDiff = imabsdiff(currentHYPRframe,averageImage);
    currentHYPRFrameMeanAbsDiff = mean(absDiff(:));

    client_handles.userData.stats_struct.currentHYPRFrameMeanAbsDiff = currentHYPRFrameMeanAbsDiff;

    set(server_1_handles.currentFrameMeanAbsErrorTag,'String',sprintf('%6.2f',currentHYPRFrameMeanAbsDiff));
    totalMeanAbsDiff = totalMeanAbsDiff + currentHYPRFrameMeanAbsDiff;
    meanTotalMeanAbsDiff = totalMeanAbsDiff/hyprFrameNumber;
    set(server_1_handles.runningMeanAbsDiffTag,'String',sprintf('%6.2f',meanTotalMeanAbsDiff));
else
    client_handles.userData.stats_struct.currentHYPRFrameMeanAbsDiff = 0;
    client_handles.userData.stats_struct.average_image_mean = 0;
    client_handles.userData.stats_struct.average_image_max  = 0;   
end

guidata(hObject,client_handles);

%
% Now see which type of plot to show on the lower right axes (the
% abs. difference axes).
%
if get(server_1_handles.abs_error_server1_btn,'Value')==1
    display_image_with_correct_gray_scale(server_1_handles.error_server1_panel_tag,...
        'histogramAxes',...
        absDiff,...
        client_handles,...
        server_1_handles);
else
    if get(server_1_handles.server_1_horiz_section_view,'Value')==1
        nCols       = size(currentC,1);
        middle_row  = round(nCols/2);
        C_strip     = currentC(middle_row,:);
        HYPR_strip  = currentHYPRframe(middle_row,:);
        h = subplot(1,1,1,'Parent',server_1_handles.error_server1_panel_tag);
        set(h,'FontSize',7);

        if get(server_1_handles.server_1_add_PC_and_PC_to_view_tag,'Value')==1
            p_strip  = current_p(middle_row,:);
            pc_strip = current_pc(middle_row,:);
            if ~get(client_handles.client_user_projection_tag,'Value')==1
                image_strip = averageImage(middle_row,:);
                y_max = max( [max(C_strip) max(HYPR_strip) max(image_strip) max(p_strip) max(pc_strip)] );
                plot(1:nCols,C_strip,'r--',...
                    1:nCols,HYPR_strip,'k',...
                    1:nCols,image_strip,'b',...
                    1:nCols,p_strip,'-.',...
                    1:nCols,pc_strip,'.');
                legend('C','hypr','image','p','pc');
            else
                y_max = max( [max(C_strip) max(HYPR_strip) max(p_strip) max(pc_strip)] );
                plot(1:nCols,C_strip,'r--',...
                    1:nCols,HYPR_strip,'k',...
                    1:nCols,p_strip,'-.',...
                    1:nCols,pc_strip,'.');
                legend('C','hypr','p','pc');
            end
        else
            if ~get(client_handles.client_user_projection_tag,'Value')==1
                image_strip = averageImage(middle_row,:);
                y_max = max( [max(C_strip) max(HYPR_strip) max(image_strip)] );
                plot(1:nCols,C_strip,'r--',...
                    1:nCols,HYPR_strip,'k',...
                    1:nCols,image_strip,'b');
                legend('C','hypr','image');
            else
                y_max = max( [max(C_strip) max(HYPR_strip) ] );
                plot(1:nCols,C_strip,'r--',...
                    1:nCols,HYPR_strip,'k');
                legend('C','hypr');
            end
        end
        if y_max>0
            set(h,'Ylim',[0 y_max+0.1*y_max]);
        end
        xlabel(h,'pixel');
        ylabel(h,'intensity');
    else  %we get here if NOT a horizontal section
        if ~get(client_handles.client_user_projection_tag,'Value')==1
            y_max = max( [max(currentC(:)) max(averageImage(:)) max(currentHYPRframe(:))]);
            y_max = y_max+.05*y_max;
            h1 = subplot(4,1,3,'Parent',server_1_handles.error_server1_panel_tag);
            plot(averageImage(:));
            set(h1,'FontSize',7);
            set(h1,'Ylim',[0 y_max]);
            set(h1,'XTick',[]);
            title(h1,'time frame');
        else
            y_max = max( [max(currentC(:)) max(currentHYPRframe(:))]);
            y_max = y_max+.05*y_max;
        end

        h1 = subplot(4,1,1,'Parent',server_1_handles.error_server1_panel_tag);
        plot(mask(:),'k.');
        set(h1,'FontSize',7);
        set(h1,'XTick',[]);
        title(h1,'mask');

        h1 = subplot(4,1,2,'Parent',server_1_handles.error_server1_panel_tag);
        plot(C(:),'m.');
        set(h1,'FontSize',7);
        set(h1,'Ylim',[0 y_max]);
        set(h1,'XTick',[]);
        title(h1,'composite');

        h1 = subplot(4,1,4,'Parent',server_1_handles.error_server1_panel_tag);
        plot(currentHYPRframe(:),'r.');
        set(h1,'FontSize',7);
        set(h1,'Ylim',[0 y_max]);
        set(h1,'XTick',[]);
        title(h1,'HYPR frame');
    end
end

%
% SNR calculations follow. see Mistreta paper
% NOT USED per discussion with Dr Pineda 7/14/08, use the noise process
% instead.
%
% SNR_hypr_paper_calculations();
%


%update_special_for_WH_disk();

guidata(hObject,client_handles);
end