%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************
function I=nma_makeRectangular(N,cx1,cy1,cx2,cy2,blackGrayLevel,whiteGrayLevel)
%function nma_makeDisk(N,r,cx,cy)
%
%function to make rectangular image
%N      %size of overall image in pixels, say 256
%cx1,cy1 %location of x,y of lower left corner
%        %all relative to center of coordinate which is assumed at 0,0
%cx2,cy2 %location of x,y of upper right corner
%
% by Nasser Abbasi July 15,2008

if nargin ~=7
    error('wrong number of arguments');
end

[errmsg,success]=verifyParamters(N,cx1,cy1,cx2,cy2);
if ~success
    error(errmsg);
end

I    = zeros(N);
I(:) = blackGrayLevel;

row1 = (-1)*cy1+ceil(N/2);
col1 = cx1+ceil(N/2);  %adjust to center it in the image
row2 = (-1)*cy2+ceil(N/2);
col2 = cx2+ceil(N/2);  %adjust to center it in the image

for row=1:N
    for col=1:N
        if row<=row1 && row>=row2 && col>=col1 && col<=col2
            I(row,col)=whiteGrayLevel;
        end
    end
end

end
%%%%%%%%%%%%%%%%%%%
% Called from above function to validate input
%
%%%%%%%%%%%%%%%%%%%
function [errmsg,success]=verifyParamters(N,cx1,cy1,cx2,cy2)
errmsg='';
success=0;

if ~isnumeric(N) ||~isnumeric(cx1)||~isnumeric(cy1) ||~isnumeric(cx2)||~isnumeric(cy2)
    errmsg='invalid argument type. Numerical values expected';
    return;
end

if ~(ceil(N) == floor(N))
    errmsg='N, size of image must be integer';
    return;
end

if ~(ceil(cx1) == floor(cx1))
    errmsg='cx, x-coordinates of disk must be integer';
    return;
end

if ~(ceil(cy1) == floor(cy1))
    errmsg='cy, y-coordinates of disk must be integer';
    return;
end

if ~(ceil(cx2) == floor(cx2))
    errmsg='cx2, x-coordinates of disk must be integer';
    return;
end

if ~(ceil(cy2) == floor(cy2))
    errmsg='cy2, y-coordinates of disk must be integer';
    return;
end

if abs(cx1)>floor(N/2) || abs(cy1)>floor(N/2)
    errmsg='coordinates outside image';
    return;
end

if abs(cx2)>floor(N/2) || abs(cy2)>floor(N/2)
    errmsg='coordinates outside image';
    return;
end


success=1;
end

