%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************

function [setOfHYPRFrames,setOfHYPRFrames_noise,status] = originalHYPR(hObject,C,C_noise,iterativeMode)
%function [setOfHYPRFrames,setOfHYPRFrames_noise,status] =
%originalHYPR(hObject,C,C_noise,iterativeMode)
%
% The Original HYPR algorithm

%===================
    function  [pcs,pcs_noise] = find_pcs_for_current_time_frame()
        pcs = zeros(client_handles.userData.sz,client_handles.userData.sz,...
            client_handles.userData.nProjectionsPerFrame);
        pcs_noise = pcs;

        pc_number = 0;

        for proj = projectionNumber:projectionNumber+client_handles.userData.nProjectionsPerFrame-1
            pc_number = pc_number + 1;
            if iterativeMode   %special case here, since composite has been updated
                angle = client_handles.userData.angles(proj);
                gc       = radon(C(:,:, hyprFrameNumber),angle);
                gc_noise = radon(C_noise(:,:, hyprFrameNumber),angle);
                pcs(:,:,pc_number) = iradon([gc gc], [angle angle],'none',client_handles.userData.sz)/2;
                pcs_noise(:,:,pc_number) = iradon([gc_noise gc_noise], [angle angle],'none',client_handles.userData.sz)/2;
            else
                pcs(:,:,pc_number)  = iradon([client_handles.userData.gc(:,proj) client_handles.userData.gc(:,proj)],...
                    [client_handles.userData.angles(proj) client_handles.userData.angles(proj)],...
                    'none',client_handles.userData.sz)/2;
                pcs_noise(:,:,pc_number)  = iradon([client_handles.userData.gc_noise(:,proj) client_handles.userData.gc_noise(:,proj)],...
                    [client_handles.userData.angles(proj) client_handles.userData.angles(proj)],...
                    'none',client_handles.userData.sz)/2;
            end
        end

        pcs(pcs<0) = 0;
    end
%===================
    function     make_one_projection()

        numberOfProjectionsInCurrentFrame = numberOfProjectionsInCurrentFrame+1;
        display_projection_vector(server_1_handles.server_1_projection_direction_axes,...
            client_handles.userData.angles(projectionNumber));

        %
        % If starting a new time frame, find the Pc for all the projections
        % now in this time frame, we can calculate the maximum of all
        % pixels to use for the threasholding later
        %
        if numberOfProjectionsInCurrentFrame == 1
            [pcs_for_current_time_frame,...
                pcs_for_current_time_frame_noise] = find_pcs_for_current_time_frame();
            % No need to scale the Pc's here, since they were obtained
            % using one-at-a-time method
            max_pc = max(pcs_for_current_time_frame(:) );
            max_pc_noise = max(pcs_for_current_time_frame(:) );
        end

        update_UI_counters_at_start_of_projection(...
            server_1_handles,...
            client_handles,...
            hyprFrameNumber,...
            numberOfProjectionsInCurrentFrame,...
            projectionNumber);

        noise_process = false;
        pc = pcs_for_current_time_frame(:,:,numberOfProjectionsInCurrentFrame);
        LR_process = false;
        [p,P_over_PC] = make_P_over_PC_ratio (...
            client_handles,...
            server_1_handles,...
            projectionNumber,...
            pc,...
            max_pc,...
            threshold,...
            noise_process,...
            LR_process);

        noise_process = true;
        [~,P_over_PC_noise] = make_P_over_PC_ratio (...
            client_handles,...
            server_1_handles,...
            projectionNumber,...
            pcs_for_current_time_frame_noise(:,:,numberOfProjectionsInCurrentFrame),...
            max_pc_noise,...
            threshold,...
            noise_process,...
            LR_process);

        mask = mask + P_over_PC;
        mask_noise = mask_noise + P_over_PC_noise;

        if numberOfProjectionsInCurrentFrame == client_handles.userData.nProjectionsPerFrame
            numberOfProjectionsInCurrentFrame = 0;

            guidata(hObject,client_handles);
            [HYPR_frame,running_psnr_vector,totalMeanAbsDiff] = ...
                nma_generate_HYPR_frame(...
                hObject,...
                hyprFrameNumber,...
                C,...
                iterativeMode,...
                mask / client_handles.userData.nProjectionsPerFrame,...
                running_psnr_vector,...
                totalMeanAbsDiff,...
                projectionNumber,...
                p,...
                pc);

            client_handles = guidata(hObject);

            if iterativeMode
                currentC_noise = C_noise(:,:, hyprFrameNumber);
            else
                currentC_noise = C_noise;
            end
            HYPR_frame_noise = mask_noise .* currentC_noise;

            setOfHYPRFrames(:,:,hyprFrameNumber)       = HYPR_frame;
            setOfHYPRFrames_noise(:,:,hyprFrameNumber) = HYPR_frame_noise;

            guidata(hObject,client_handles);
            find_SNR_and_update_log_file(...
                hObject,...
                HYPR_frame_noise,...
                currentC_noise);

            client_handles = guidata(hObject);
            client_handles.userData.stats_array{hyprFrameNumber}=...
                client_handles.userData.stats_struct;

            guidata(hObject,client_handles);

            hyprFrameNumber = hyprFrameNumber + 1;
            mask            = zeros(client_handles.userData.sz);
            mask_noise      = zeros(client_handles.userData.sz);
            
            % 
            % Now that we have one frame processed, check if we
            % are doing sliding window. If so, make new C's
            % But make sure we not in iterative mode. Sliding window
            % only applied to non-iterative algorithms. There is no 
            % reason not to apply it to iterative algorithms as well,
            % except at this time we need to learn how to walk before we
            % run. May be in the future I could think about using sliding
            % window in I-HYPR and IW-HYPR but not today...
            %
            if ~iterativeMode
                if get(client_handles.client_sliding_window_tag,'Value')==1
                    centered_frame_number = hyprFrameNumber;
                    [C,C_noise] = get_composite_image(...
                        client_handles,...
                        centered_frame_number);
                end
            end

        else
            if get(server_1_handles.turnOnProjectionUpdatesTag,'Value')==1
                plot_the_mask(...
                    server_1_handles,...
                    client_handles,...
                    mask/numberOfProjectionsInCurrentFrame);
                update_intensity_profile(hObject,p,pc);
            end
        end
        drawnow;
    end
%===================

%This function is called multiple times as the kernel to I-HYPR

client_handles   = guidata(hObject);
server_1_handles = guidata(client_handles.userData.server_1_handle);

mask = zeros(client_handles.userData.sz);
mask_noise = zeros(client_handles.userData.sz);
running_psnr_vector = zeros(client_handles.userData.nTimeFrames,1);
log_file_initialization_per_algorithm(client_handles.userData.fid);

totalMeanAbsDiff = 0;

%
% Allocate space to keep the HYPR images
%
setOfHYPRFrames = zeros(client_handles.userData.sz,...
    client_handles.userData.sz,...
    client_handles.userData.nTimeFrames);

setOfHYPRFrames_noise = setOfHYPRFrames;

pcs_for_current_time_frame = [];
pcs_for_current_time_frame_noise = [];
max_pc = 0;
max_pc_noise = 0;

%
% make a finite state machine to handle logic server1 states
% This is needed to handle the play,pause,step, etc...
% This is much cleaner method than using a bunch of if/then/elses
%

%
% These below are the possible states the server will flip between
%
PAUSE=0; PLAY=1; MOVE_TO_NEXT_PROJECTION=2;
MOVE_TO_END_OF_FRAME=3; CANCEL=4;

projectionNumber = 1;
numberOfProjectionsInCurrentFrame = 0;
hyprFrameNumber = 1;
NORMAL  = 1;
ABORTED = 0;

threshold = str2double(get(client_handles.client_threshold_tag,'String'));
status = NORMAL;

%
% Start the main loop. We will stay in this as long as we have not reached
% end of the projections or user has not interrupted us by clicking on the
% 'go back to client' button
%
while projectionNumber<=client_handles.userData.totalNumberOfProjections ...
        && status == NORMAL

    client_handles  = guidata(hObject);
    switch client_handles.interrupt_state.server1_state
        case PAUSE
            set(server_1_handles.statusTag,'String','Click PLAY to continue....');            
            pause(.1);
        case MOVE_TO_NEXT_PROJECTION
            PARTIAL = false;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'off');

            client_handles.interrupt_state.server1_state = PAUSE;
            guidata(hObject,client_handles);
            make_one_projection();
            projectionNumber = projectionNumber + 1;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'on');

        case MOVE_TO_END_OF_FRAME
            PARTIAL = false;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'off');

            number_of_projections_to_end_of_current_frame = ...
                client_handles.userData.nProjectionsPerFrame-numberOfProjectionsInCurrentFrame;

            for k=1:number_of_projections_to_end_of_current_frame
                make_one_projection();
                projectionNumber = projectionNumber + 1;
            end

            client_handles.interrupt_state.server1_state = PAUSE;
            guidata(hObject,client_handles);
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'on');

        case PLAY
            PARTIAL = true;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'off');
            make_one_projection();
            projectionNumber = projectionNumber + 1;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'on');

        case CANCEL
            status = ABORTED;
            PARTIAL = true;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'off');
    end
end

if status == NORMAL
    %    axes(server_1_handles.temporalProfileAxes);
    %    hold(server_1_handles.temporalProfileAxes,'off');

    set(server_1_handles.statusTag,'String','Completed generating HYPR frames.');
    client_handles.userData.averagedPSNR    = mean(running_psnr_vector);
    client_handles.userData.averagedAbsDiff = totalMeanAbsDiff/client_handles.userData.nTimeFrames;

    update_log_file_with_stats(client_handles);

    client_handles.interrupt_state.server_1_completed_normal = 1;
else
    client_handles.interrupt_state.server_1_completed_normal = 0;
    set(server_1_handles.statusTag,'String','Aborted....');
end
guidata(hObject,client_handles);

end

