%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************

function script_effect_of_filtered()

%Showing the effect of filtered backprojection
%vs non-filtered backprojection

close all;
clear all;

if 0

    fileName = 'monkey.jpg';
    A        = double(imread(fileName,'jpg'));
    sz = size(A,1);
    totalNumberOfProjections=128;
    userImages=zeros(sz,sz,totalNumberOfProjections);

    for i = 1:totalNumberOfProjections
        userImages(:,:,i) =  A;
    end

end

if 1
    data=load('paperclip_phantom.mat');
    userImages = data.im;

    sz = size(userImages,1);
    totalNumberOfProjections = 64;
end

if 0
    radius      = 10;   %in pixels radius of disk
    minGrayLevel = 1;
    maxGrayLevel = 128;
    sz = 128;
    totalNumberOfProjections=128;

    cy   = linspace(-round(sz/2),round(sz/2)-1,totalNumberOfProjections);
    cx1  = 40;
    cx2  = -40;

    userImages = zeros(sz,sz,totalNumberOfProjections);

    for i = 1:totalNumberOfProjections
        A = nma_makeDisk(sz ,radius,...
            round(cx1),round(cy(i)),minGrayLevel,maxGrayLevel);
        A = A + nma_makeDisk(sz ,radius,round(cx2),round(cy(i)),...
            minGrayLevel,maxGrayLevel);

        userImages(:,:,i) =  A;
    end
end

figure(1);

angles = linspace(0,180,totalNumberOfProjections);
fbp=zeros(sz);
bp =zeros(sz);

%
% Make on quick forward projection just to get the size of it
%
projection=radon(userImages(:,:,1),angles(1));

%
% now we know how large projection is, allocate memory and make projections
%
T=zeros(length(projection),totalNumberOfProjections);
for i=1:totalNumberOfProjections
    T(:,i) = radon(userImages(:,:,i),angles(i));
end



for i=1:totalNumberOfProjections

    g     = T(:,i);
    theta = angles(i);

    h=subplot(2,2,1);
    imshow(userImages(:,:,i),'Parent',h,'InitialMagnification',150);
    title(h,'Original');

    fbp = fbp + iradon([g g],[theta theta],sz)/2;
    h=subplot(2,2,2);
    imshow(fbp,[],'Parent',h,'InitialMagnification',150);
    title(h,sprintf('Filtered, add one at time. i=%d',i));
    xlabel(h,sprintf('angle=%4.2f',theta));

    if i==1
        t = iradon([g g],[theta theta],'none',sz)/2;
    else
        t = iradon(T(:,1:i),angles(1:i),'none',sz);
    end
    h=subplot(2,2,3);
    imshow(t,'Parent',h,'InitialMagnification',150);
    title(h,sprintf('UnFiltered'));

    if i==1
        t = iradon([g g],[theta theta],sz)/2;
    else
        t = iradon(T(:,1:i),angles(1:i),sz);
    end
    h = subplot(2,2,4);
    imshow(t,'Parent',h,'InitialMagnification',150);
    title(h,sprintf('Filtered, all at once'));

    drawnow;
    pause(.1);
end
end

