%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************

% 
% A matlab 20007a script to illustrate that doing
% backprojections (filtered) on a set of projections all at once
% is not the same as backprojecting each projection and adding the sum

%by Nasser Abbasi

close all;
clear all;

%
% load the image, and make some angles to project the image at
%
sz           = 256;
ph           = phantom(256);
nProjections = 128;
angles       = linspace(0,180,nProjections);
fbp          = zeros(sz); %allocate FBP image to accumlate backprojection

%
% Fire it up. Find forward projections, and backprojections
%
g = radon(ph,angles);
figure(1); 

h = subplot(2,2,1);
imshow(ph,[],'Parent',h,'InitialMagnification',150);  
colormap(jet);
title(h,'Original image','FontSize',7);
h = colorbar();  
set(h,'FontSize',7);   

% Let the fun begin
for i=1:nProjections

    theta = angles(i);    
    
    subplot(2,2,2);
    plot(g(:,i)); 
    title(sprintf('projection %d, angle=%4.2f',i,theta),'FontSize',7);
    h = colorbar();      set(h,'FontSize',7);   
       
    % Make backprojection, and add to accumlator image
    fbp = fbp + iradon([g(:,i) g(:,i)],[theta theta],'none',sz)/2;
    h   = subplot(2,2,3);
    imshow(fbp,[],'Parent',h,'InitialMagnification',150);  
    title(h,sprintf('Filtered, add one at time.'));
    xlabel(h,sprintf('mean=%5.3f',mean(fbp(:))),'FontSize',8);
    h = colorbar();     set(h,'FontSize',7);       

    %now make backprojection, all at once, watch for initial case
    if i==1
       t = iradon([g(:,i) g(:,i)],[theta theta],'none',sz)/2;
    else
       t = iradon(g(:,1:i),angles(1:i),'none',sz);        
    end
    h = subplot(2,2,4);
    imshow(t,[],'Parent',h,'InitialMagnification',150);  
    colormap(jet);    
    title(h,sprintf('Filtered, all at once'),'FontSize',7);
    xlabel(h,sprintf('mean=%5.3f',mean(t(:))),'FontSize',8);    
    h = colorbar();     set(h,'FontSize',7);       
    
    drawnow;
end

    % Fix the one at a time case
    h        = subplot(2,2,3);
    my_magic_correction = fbp/(nProjections/2);
    imshow(my_magic_correction,[],'Parent',h,'InitialMagnification',150);  
    title(h,sprintf('Filtered, add one at time, adjusted.'));
    xlabel(h,sprintf('mean=%5.3f',mean(my_magic_correction(:))),'FontSize',8);
    h = colorbar();     set(h,'FontSize',7);       
    colormap(jet);       
