%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************

function  update_log_file_with_stats(h)
%function  update_log_file_with_stats(h)
%
%This is the entry called from the simulator at the end
%of ALL HYPR frames reconstruction to log all the statistics
%
%i.e. at the end of the HYPR algorithms. In an iterative HYPR, this will
%therefor be called by N times, where N is the number of I-HYPR iterations.
%
%Nasser Abbasi

data = cell2mat(h.userData.stats_array);
number_of_frames_per_table = 5;
number_of_frames_remaining = h.userData.nTimeFrames;
current_frame = 1;
more_to_do    = true;
fid           = h.userData.fid;

while more_to_do
    if number_of_frames_remaining <= number_of_frames_per_table
        emit_table(data(current_frame:end),fid);
        more_to_do = false;
    else
        emit_table(data(current_frame:current_frame+number_of_frames_per_table-1),fid);
        current_frame = current_frame + number_of_frames_per_table;
        number_of_frames_remaining = number_of_frames_remaining - number_of_frames_per_table;
        updateLogFile(sprintf('\n'),fid);
    end
end

updateLogFile(sprintf('\n'),fid);
updateLogFile(sprintf('------ T A B L E    E N D S   H E R E  --------------\n'),fid);
updateLogFile(sprintf('\n Cummulative statistics begins>> \n'),fid);

updateLogFile(sprintf('Mean relative RMSE for this test is %f\n',...
    mean([data(:).RMSE])),fid);
updateLogFile(sprintf('meadian relative RMSE for this test is %f\n',...
    median([data(:).RMSE])),fid);
updateLogFile(sprintf('Standard deviation of relative RMSE for this test is %f\n',...
    std([data(:).RMSE])),fid);

end

function emit_table(data,fid)

msg=sprintf('1)frame\t\t');
updateLogFile(msg,fid);
msg=sprintf('%d\t\t',data(:).frame);
updateLogFile(msg,fid);
msg=sprintf('\n');
updateLogFile(msg,fid);

msg=sprintf('2)RMSE\t\t');
updateLogFile(msg,fid);
msg=sprintf('%5.4f\t',data(:).RMSE);
updateLogFile(msg,fid);
msg=sprintf('\n');
updateLogFile(msg,fid);


msg=sprintf('3)AbsDiff\t\t');
updateLogFile(msg,fid);
msg=sprintf('%5.4f\t',data(:).currentHYPRFrameMeanAbsDiff);
updateLogFile(msg,fid);
msg=sprintf('\n');
updateLogFile(msg,fid);


msg=sprintf('4)noise hypr mse\t');
updateLogFile(msg,fid);
msg=sprintf('%5.4f\t',data(:).noise_hypr_mse);
updateLogFile(msg,fid);
msg=sprintf('\n');
updateLogFile(msg,fid);


msg=sprintf('5)noise C mse\t');
updateLogFile(msg,fid);
msg=sprintf('%5.4f\t',data(:).noise_C_mse);
updateLogFile(msg,fid);
msg=sprintf('\n');
updateLogFile(msg,fid);


msg=sprintf('6)noise hypr mean\t');
updateLogFile(msg,fid);
msg=sprintf('%5.4f\t',data(:).noise_hypr_mean);
updateLogFile(msg,fid);
msg=sprintf('\n');
updateLogFile(msg,fid);

msg=sprintf('7)noise C mean\t');
updateLogFile(msg,fid);
msg=sprintf('%5.4f\t',data(:).noise_C_mean);
updateLogFile(msg,fid);
msg=sprintf('\n');
updateLogFile(msg,fid);

msg=sprintf('8)noise hypr std\t');
updateLogFile(msg,fid);
msg=sprintf('%5.4f\t',data(:).noise_hypr_std);
updateLogFile(msg,fid);
msg=sprintf('\n');
updateLogFile(msg,fid);


msg=sprintf('9)noise C std\t');
updateLogFile(msg,fid);
msg=sprintf('%5.4f\t',data(:).noise_C_std);
updateLogFile(msg,fid);
msg=sprintf('\n');
updateLogFile(msg,fid);

msg=sprintf('10)hypr mean\t');
updateLogFile(msg,fid);
msg=sprintf('%5.4f\t',data(:).hypr_mean);
updateLogFile(msg,fid);
msg=sprintf('\n');
updateLogFile(msg,fid);

msg=sprintf('11)hypr max\t\t');
updateLogFile(msg,fid);
msg=sprintf('%5.4f\t',data(:).hypr_max);
updateLogFile(msg,fid);
msg=sprintf('\n');
updateLogFile(msg,fid);


msg=sprintf('12)avg image mean\t');
updateLogFile(msg,fid);
msg=sprintf('%5.4f\t',data(:).average_image_mean);
updateLogFile(msg,fid);
msg=sprintf('\n');
updateLogFile(msg,fid);

msg=sprintf('13)avg image max\t');
updateLogFile(msg,fid);
msg=sprintf('%5.4f\t',data(:).average_image_max);
updateLogFile(msg,fid);
msg=sprintf('\n');
updateLogFile(msg,fid);


end