%****************************************************
%       Copyright (C) 2008 Nasser Abbasi
%  Free to use and modify for academic and research only
%***************************************************

function [setOfHYPRFrames,setOfHYPRFrames_noise,status] = wrightHuangHYPR(hObject,C,C_noise,iterativeMode)

%======================
    function [p,pc,p_noise,pc_noise] = find_p_and_pc(projectionNumber)

        i = projectionNumber;
        FILTERED = false;
        MODE = 0;
        noise_process = false;
        from = i - numberOfProjectionsInCurrentFrame + 1;
        to   = i;

        p = doBackProjection( client_handles ,...
            FILTERED,...
            from,...
            to,...
            MODE,...
            noise_process);

        noise_process = true;
        p_noise = doBackProjection( client_handles ,...
            FILTERED,...
            from,...
            to,...
            MODE,...
            noise_process);

        if iterativeMode == 1
            composite = C(:,:, hyprFrameNumber);
            composite_noise = C_noise(:,:, hyprFrameNumber);
            gc = zeros(numel(client_handles.userData.gc(:,1)) ,...
                client_handles.userData.nProjectionsPerFrame);
            gc_noise =  zeros(numel(client_handles.userData.gc_noise(:,1)) ,...
                client_handles.userData.nProjectionsPerFrame);

            j = 0;
            for k = from : to
                j = j + 1;
                gc(:,j) = radon(composite,client_handles.userData.angles(k));
                gc_noise(:,j) = radon(composite_noise,client_handles.userData.angles(k));
            end

            T      = zeros(to-from+1,2);
            T(:,1) = client_handles.userData.angles(from : to);
            T(:,2) = 1:to-from+1;
            T      = sortrows(T);

            pc = iradon(gc(:,T(:,2)),T(:,1),'none',client_handles.userData.sz);
            pc(pc<0) = 0;
            pc_noise = iradon(gc_noise(:,T(:,2)),T(:,1),'none',client_handles.userData.sz);
            pc_noise(pc_noise<0) = 0;
        else
            FILTERED = false;
            MODE = 1;
            noise_process = false;
            from = i - numberOfProjectionsInCurrentFrame + 1;
            to   = i;

            pc = doBackProjection( client_handles ,...
                FILTERED,...
                from,...
                to,...
                MODE,...
                noise_process);

            noise_process = true;
            pc_noise = doBackProjection( client_handles ,...
                FILTERED,...
                from,...
                to,...
                MODE,...
                noise_process);
        end
    end

%======================
    function make_one_projection()

        i = projectionNumber;
        numberOfProjectionsInCurrentFrame = numberOfProjectionsInCurrentFrame+1;
        display_projection_vector(server_1_handles.server_1_projection_direction_axes,...
            client_handles.userData.angles(projectionNumber));

        if get(server_1_handles.turnOnProjectionUpdatesTag,'Value')==1
            plot(server_1_handles.server_1_projection_on_P_axes,...
                client_handles.userData.g(:,projectionNumber));
            if iterativeMode == 1
                gc = radon(C(:,:, hyprFrameNumber),client_handles.userData.angles(projectionNumber));
            else
                gc = radon(C,client_handles.userData.angles(projectionNumber));
            end
            plot(server_1_handles.server_1_projection_on_PC_axes,gc);
        end

        update_UI_counters_at_start_of_projection(...
            server_1_handles,...
            client_handles,...
            hyprFrameNumber,...
            numberOfProjectionsInCurrentFrame,...
            projectionNumber);

        HYPRcommonLoopProcess(client_handles,server_1_handles,i);

        [p,pc,p_noise,pc_noise] = find_p_and_pc(projectionNumber);

        if numberOfProjectionsInCurrentFrame == client_handles.userData.nProjectionsPerFrame
            numberOfProjectionsInCurrentFrame = 0;

            if get(server_1_handles.turnOnProjectionUpdatesTag,'Value')==1
                axes(server_1_handles.currentPCAxes);
                imshow(pc,[]); %colorbar;
                setXandTitleLabels(sprintf('PC[%d-%d],mean=%6.2f,max=%6.2f',...
                    i-client_handles.userData.nProjectionsPerFrame+1,i,mean(pc(:)),max(pc(:))),...
                    '',7,...
                    client_handles.userData.xlabelOffset);

                axes(server_1_handles.currentPAxes);
                imshow(p,[]); %colorbar;

                setXandTitleLabels(sprintf('P[%d-%d],mean=%6.2f,max=%6.2f',...
                    i-client_handles.userData.nProjectionsPerFrame+1,i,mean(p(:)),max(p(:))),...
                    '',7,...
                    client_handles.userData.xlabelOffset);
            end

            set(server_1_handles.currentFrameNumberTag,'String',...
                sprintf('%d',hyprFrameNumber));

            %make the mask
            if get(client_handles.client_set_pc_zero_btn,'Value')==1
                mask              = p./pc;
                mask(isnan(mask)) = 0;
                mask(isinf(mask)) = 0;
            else
                set_value         = threshold*0.01*max(pc(:));
                pc(pc<set_value)  = set_value;
                mask              = p./pc;
                mask(isnan(mask)) = 0;
            end

            if get(client_handles.client_set_pc_zero_btn,'Value')==1
                mask_noise                    = p_noise./pc_noise;
                mask_noise(isnan(mask_noise)) = 0;
                mask_noise(isinf(mask_noise)) = 0;
            else
                set_value                     = threshold*0.01*max(pc_noise(:));
                pc_noise(pc_noise<set_value)  = set_value;
                mask_noise                    = p_noise./pc_noise;
                mask_noise(isnan(mask_noise)) = 0;
            end

            guidata(hObject,client_handles);
            [currentHYPRframe,running_psnr_vector,...
                totalMeanAbsDiff] = ...
                nma_generate_HYPR_frame(...
                hObject,...
                hyprFrameNumber,...
                C,...
                iterativeMode,...
                mask ,...
                running_psnr_vector,...
                totalMeanAbsDiff,...
                i,...
                p,...
                pc);
            client_handles = guidata(hObject);

            setOfHYPRFrames(:,:,hyprFrameNumber) = currentHYPRframe;

            if iterativeMode
                currentC_noise = C_noise(:,:, hyprFrameNumber);
            else
                currentC_noise = C_noise;
            end
            HYPR_frame_noise = mask_noise .* currentC_noise;
            setOfHYPRFrames_noise(:,:,hyprFrameNumber) = HYPR_frame_noise;

            guidata(hObject,client_handles);
            find_SNR_and_update_log_file(...
                hObject,...
                HYPR_frame_noise,...
                currentC_noise);

            client_handles = guidata(hObject);
            client_handles.userData.stats_array{hyprFrameNumber}=...
                client_handles.userData.stats_struct;
            guidata(hObject,client_handles);

            hyprFrameNumber = hyprFrameNumber + 1;
            % 
            % Now that we have one frame processed, check if we
            % are doing sliding window. If so, make new C's. See comment
            % also in originalHYPR.m
            %
            if ~iterativeMode
                if get(client_handles.client_sliding_window_tag,'Value')==1
                    centered_frame_number = hyprFrameNumber;
                    [C,C_noise] = get_composite_image(...
                        client_handles,...
                        centered_frame_number);
                end
            end            
            
        else
            update_intensity_profile(hObject,p,pc);
        end
        drawnow;
    end
%==================

client_handles   = guidata(hObject);
server_1_handles = guidata(client_handles.userData.server_1_handle);

mask                = zeros(client_handles.userData.sz);
mask_noise = zeros(client_handles.userData.sz);

running_psnr_vector = zeros(client_handles.userData.nTimeFrames,1);

totalMeanAbsDiff = 0;

setOfHYPRFrames = zeros(client_handles.userData.sz,...
    client_handles.userData.sz,...
    client_handles.userData.nTimeFrames);
setOfHYPRFrames_noise = setOfHYPRFrames;

threshold = str2double(get(client_handles.client_threshold_tag,'String'));

log_file_initialization_per_algorithm(client_handles.userData.fid);

% make a finite state machine to handle logic
%server1 states
PAUSE=0; PLAY=1; MOVE_TO_NEXT_PROJECTION=2;
MOVE_TO_END_OF_FRAME=3; CANCEL=4;

projectionNumber = 1;
numberOfProjectionsInCurrentFrame = 0;
hyprFrameNumber = 1;
NORMAL  = 1;
ABORTED = 0;

status = NORMAL;

while projectionNumber<=client_handles.userData.totalNumberOfProjections ...
        && status == NORMAL

    client_handles  = guidata(hObject);
    switch client_handles.interrupt_state.server1_state
        case PAUSE
            set(server_1_handles.statusTag,'String','Click PLAY to continue....');                        
            pause(.1);
        case MOVE_TO_NEXT_PROJECTION
            PARTIAL = false;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'off');

            client_handles.interrupt_state.server1_state = PAUSE;
            guidata(hObject,client_handles);
            make_one_projection();
            projectionNumber = projectionNumber + 1;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'on');

        case MOVE_TO_END_OF_FRAME
            PARTIAL = false;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'off');

            number_of_projections_to_end_of_current_frame = ...
                client_handles.userData.nProjectionsPerFrame-numberOfProjectionsInCurrentFrame;

            for n=1:number_of_projections_to_end_of_current_frame
                make_one_projection();
                projectionNumber = projectionNumber + 1;
            end

            client_handles.interrupt_state.server1_state = PAUSE;
            guidata(hObject,client_handles);

            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'on');

        case PLAY
            PARTIAL = true;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'off');
            make_one_projection();
            projectionNumber = projectionNumber + 1;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'on');

        case CANCEL
            status = ABORTED;
            PARTIAL = false;
            set_all_server1_navagation_buttons_to(server_1_handles,PARTIAL,'off');
    end
end

if status == NORMAL
    set(server_1_handles.statusTag,'String','Completed generating HYPR frames.');
    client_handles.userData.averagedPSNR    = mean(running_psnr_vector);
    client_handles.userData.averagedAbsDiff = totalMeanAbsDiff/client_handles.userData.nTimeFrames;

    update_log_file_with_stats(client_handles);

    client_handles.interrupt_state.server_1_completed_normal = 1;
else
    client_handles.interrupt_state.server_1_completed_normal = 0;
    set(server_1_handles.statusTag,'String','Aborted....');
end
guidata(hObject,client_handles);

end
